import json

from django.contrib import admin, messages
from django.contrib.admin import helpers
from django.template.response import TemplateResponse
from django.utils.html import format_html

from common.admin import AuditMixin

from .controllers import rerun_logged_task
from .models import LoggedTask


class LoggedTaskAdmin(AuditMixin, admin.ModelAdmin):

    list_display = ('id', 'queue', 'task_name', 'created_at', 'args', 'kwargs',
                    'status')
    list_filter = ('task_name', 'status')

    fields = readonly_fields = ('id', 'queue', 'created_at', 'task_name', 'admin_args',
                                'admin_kwargs', 'admin_stacktrace', 'status')
    ordering = ('-created_at',)

    actions = ['retry_logged_tasks']

    def retry_logged_tasks(logged_task_admin, request, queryset):
        opts = logged_task_admin.model._meta

        if request.method == 'POST':
            for logged_task in queryset:
                rerun_logged_task(logged_task)
            count = queryset.count()
            message = 'Retried {} Logged Task items'.format(count)
            logged_task_admin.message_user(request, message, messages.SUCCESS)
            # Return None to display the change list page again.
            return None

        context = {
            'title': 'Retry Tasks?',
            'queryset': queryset,
            'opts': opts,
            'action_checkbox_name': helpers.ACTION_CHECKBOX_NAME,
            'media': logged_task_admin.media,
        }

        return TemplateResponse(
            request,
            'admin/retry_task_confirmation.html',
            context=context,
            current_app=logged_task_admin.admin_site.name,
        )

    def admin_stacktrace(self, logged_task):
        return format_html('<br/><pre>{}</pre>', logged_task.stacktrace)

    def admin_args(self, logged_task):
        args = json.loads(logged_task.args)
        return format_html('<br/><pre>{}</pre>', json.dumps(args, indent=4))

    def admin_kwargs(self, logged_task):
        kwargs = json.loads(logged_task.kwargs)
        return format_html('<br/><pre>{}</pre>', json.dumps(kwargs, indent=4))

    retry_logged_tasks.short_description = 'Retry Selected Tasks'


admin.site.register(LoggedTask, LoggedTaskAdmin)



# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2016-09-20 21:28
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='LoggedTask',
            fields=[
                ('id', models.CharField(max_length=255, primary_key=True, serialize=False, unique=True)),
                ('task_name', models.CharField(max_length=2056)),
                ('status', models.CharField(max_length=50)),
                ('args', models.TextField(default='[]')),
                ('kwargs', models.TextField(default='{}')),
                ('stacktrace', models.TextField(blank=True)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
        ),
    ]



import json

from django.db.models import signals
from django.test import TestCase

from mbq.ranch.constants import TaskStatus
from mbq.ranch.controllers import persist_task, rerun_logged_task
from mbq.ranch.exceptions import TaskNotFound
from mbq.ranch.models import LoggedTask

from mock import Mock, patch


class TaskManagementTests(TestCase):

    def setUp(self):
        signals.post_save.disconnect(
            sender=LoggedTask,
            dispatch_uid='logged_task_save'
        )
        signals.post_delete.disconnect(
            sender=LoggedTask,
            dispatch_uid='logged_task_delete'
        )

    def test_persist_failed_task(self):
        LoggedTask.objects.all().delete()

        persist_task(
            'task_id',
            'queue_name',
            'task_name',
            [1, 2, 3],
            {'foo': 1, 'bar': 2},
            'stacktrace',
            TaskStatus.FAILURE,
        )

        logged_task = LoggedTask.objects.filter(
            id='task_id',
            task_name='task_name',
            stacktrace='stacktrace',
            status=TaskStatus.FAILURE,
        ).first()

        self.assertIsNotNone(logged_task)
        self.assertEqual(
            json.loads(logged_task.args),
            [1, 2, 3],
        )
        self.assertEqual(
            json.loads(logged_task.kwargs),
            {'foo': 1, 'bar': 2},
        )

    @patch('mbq.ranch.controllers.import_module')
    def test_rerun_task_with_params(self, module_import_mock):
        module_mock = Mock()
        module_import_mock.return_value = module_mock

        logged_task = persist_task(
            'task_id',
            'queue_name',
            'some.module.lookup.and.thefunction',
            ['a', 'b'],
            {'fizz': 'buzz'},
            'stacktrace',
            TaskStatus.FAILURE,
        )
        rerun_logged_task(logged_task)

        module_import_mock.assert_called_once_with(
            'some.module.lookup.and'
        )
        module_mock.thefunction.delay.assert_called_once_with(
            'a',
            'b',
            fizz='buzz'
        )
        self.assertFalse(LoggedTask.objects.filter(pk=logged_task.pk).exists())

    @patch('mbq.ranch.controllers.import_module')
    def test_rerun_task_with_no_params(self, module_import_mock):
        module_mock = Mock()
        module_import_mock.return_value = module_mock

        logged_task = persist_task(
            'task_id',
            'queue_name',
            'some.module.lookup.and.thefunction',
            [],
            {},
            'stacktrace',
            TaskStatus.FAILURE,
        )
        rerun_logged_task(logged_task)

        module_import_mock.assert_called_once_with(
            'some.module.lookup.and'
        )
        module_mock.thefunction.delay.assert_called_once_with()

        self.assertFalse(LoggedTask.objects.filter(pk=logged_task.pk).exists())

    @patch('mbq.ranch.controllers.import_module')
    def test_rerun_task_not_found(self, module_import_mock):
        module_mock = Mock()
        module_import_mock.return_value = module_mock

        del module_mock.somefunction

        logged_task = persist_task(
            'task_id',
            'queue_name',
            'somemodule.somefunction',
            ['a', 'b'],
            {'fizz': 'buzz'},
            'stacktrace',
            TaskStatus.FAILURE,
        )

        with self.assertRaises(TaskNotFound):
            rerun_logged_task(logged_task)



import os

import mbq.metrics

import dj_database_url


SECRET_KEY = 'fake-key'
DEBUG = True
RANCH = {
    'env': 'Test',
    'service': 'test-service',
}

database_url = os.environ.get('DATABASE_URL', 'mysql://root:@mysql:3306/ranchdb')
DATABASES = {
    'default': dj_database_url.parse(database_url),
}

INSTALLED_APPS = [
    'mbq.ranch',
]

USE_TZ = True

mbq.metrics.init()



__author__ = 'Managed by Q, Inc.',
__author_email__ = 'open-source@managedbyq.com'
__description__ = 'MBQ Ranch'
__license__ = 'Apache 2.0'
__title__ = 'mbq.ranch'
__url__ = 'https://github.com/managedbyq/mbq.ranch'
__version__ = '0.0.1'



# -*- coding: utf-8 -*-
# Generated by Django 1.11.15 on 2018-10-01 17:56
from django.db import migrations, models

class Migration(migrations.Migration):

    dependencies = [
        ('ranch', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='loggedtask',
            name='queue',
            field=models.CharField(max_length=128, null=True),
        ),
    ]



from django.apps import AppConfig
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

from mbq import metrics


class RanchConfig(AppConfig):
    name = 'mbq.ranch'
    verbose_name = 'Ranch'

    def ready(self):
        env = settings.RANCH.get('env')
        service = settings.RANCH.get('service')

        if not all([env, service]):
            raise ImproperlyConfigured(
                'mbq.ranch must be configured with env and service parameters.'
            )

        self.module._collector = metrics.Collector(
            namespace='mbq.ranch',
            tags={
                'env': env,
                'service': service,
            },
        )

        from . import signal_handlers  # noqa
